"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const lodash_1 = require("lodash");
const path = require("path");
const mergePackages_1 = require("./mergePackages");
const readPackageDeps = lodash_1.memoize((fileDir) => {
    try {
        const packageContent = fs.readFileSync(path.join(fileDir, "package.json"), "utf8");
        try {
            return mergePackages_1.normalizedPackage(JSON.parse(packageContent));
        }
        catch (e) {
        }
    }
    catch (e) {
        const nextDir = path.dirname(fileDir);
        if (nextDir !== fileDir) {
            return readPackageDeps(nextDir);
        }
    }
    return;
});
function maybeReadPackage(pFileDir) {
    try {
        const lPackageContent = fs.readFileSync(path.join(pFileDir, "package.json"), "utf8");
        try {
            return mergePackages_1.normalizedPackage(JSON.parse(lPackageContent));
        }
        catch (e) {
        }
    }
    catch (e) {
    }
    return;
}
function getIntermediatePaths(fileDir, baseDir) {
    const retval = [];
    let intermediate = fileDir;
    while (intermediate !== baseDir &&
        intermediate !== path.dirname(intermediate)) {
        retval.push(intermediate);
        intermediate = path.dirname(intermediate);
    }
    retval.push(baseDir);
    return retval;
}
const readPackageDepsCombined = lodash_1.memoize((fileDir, baseDir) => {
    if (!fileDir.startsWith(baseDir) || baseDir.endsWith(path.sep)) {
        throw new Error(`Unexpected Error: Unusal baseDir passed to package reading function: '${baseDir}'\n` +
            `Please file a bug: https://github.com/sverweij/dependency-cruiser/issues/new?template=bug-report.md` +
            `&title=Unexpected Error: Unusal baseDir passed to package reading function: '${baseDir}'`);
    }
    const retval = getIntermediatePaths(fileDir, baseDir).reduce((all, current) => mergePackages_1.mergePackages(all, maybeReadPackage(current)), {});
    return Object.keys(retval).length > 0 ? retval : undefined;
});
exports.default = (fileDir, baseDir, combinedDependencies = false) => {
    if (combinedDependencies) {
        return readPackageDepsCombined(fileDir, baseDir);
    }
    else {
        return readPackageDeps(fileDir);
    }
};
