"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const normalize = require("normalize-package-data");
exports.normalizedPackage = (pkg) => {
    normalize(pkg);
    return pkg;
};
function normalizePackageKeys(pkg) {
    const retval = pkg;
    if (pkg.bundleDependencies) {
        pkg.bundledDependencies = lodash_1.clone(pkg.bundleDependencies);
        Reflect.deleteProperty(pkg, "bundleDependencies");
    }
    return retval;
}
function mergeDependencyKey(closestDependencyKey, furtherDependencyKey) {
    return { ...furtherDependencyKey, ...closestDependencyKey };
}
function mergeDependencyArray(closestDependencyKey, furtherDependencyKey) {
    return lodash_1.uniq(closestDependencyKey.concat(furtherDependencyKey));
}
function isDependencyKey(key) {
    return key.endsWith("ependencies");
}
function getDependencyKeys(pkg) {
    return Object.keys(pkg).filter(isDependencyKey);
}
function getJointDependencyKeys(closestPackage, furtherPackage) {
    return lodash_1.uniq(getDependencyKeys(closestPackage).concat(getDependencyKeys(furtherPackage)));
}
exports.mergePackages = (closestPackage, furtherPackage) => getJointDependencyKeys(normalizePackageKeys(closestPackage), normalizePackageKeys(furtherPackage))
    .map(pKey => ({
    key: pKey,
    value: pKey.startsWith("bundle")
        ? mergeDependencyArray(lodash_1.get(closestPackage, pKey, []), lodash_1.get(furtherPackage, pKey, []))
        : mergeDependencyKey(lodash_1.get(closestPackage, pKey, {}), lodash_1.get(furtherPackage, pKey, {})),
}))
    .reduce((pJoinedObject, pJoinedKey) => {
    pJoinedObject[pJoinedKey.key] = pJoinedKey.value;
    return pJoinedObject;
}, {});
