"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const resolve_1 = require("resolve");
const localNpmHelpers = require("./localNpmHelpers");
const npm2depType = {
    dependencies: "npm",
    devDependencies: "npm-dev",
    optionalDependencies: "npm-optional",
    peerDependencies: "npm-peer",
};
function determineNpmDependencyTypes(moduleName, packageDependencies) {
    let retval = ["npm-unknown"];
    if (Boolean(packageDependencies)) {
        retval = Object.keys(packageDependencies)
            .filter(key => key.includes("ependencies") && packageDependencies[key].hasOwnProperty(moduleName))
            .map(key => npm2depType[key] || "npm-no-pkg");
        retval = retval.length === 0 ? ["npm-no-pkg"] : retval;
    }
    return retval;
}
function dependencyIsBundled(pModule, pPackageDeps) {
    if (Boolean(pPackageDeps)) {
        const bundledDependencies = pPackageDeps.bundledDependencies || pPackageDeps.bundleDependencies || {};
        if (bundledDependencies) {
            return Object.keys(bundledDependencies).some(dependencyName => dependencyName === pModule);
        }
    }
    return false;
}
function determineNodeModuleDependencyTypes(pModuleName, pPackageDeps, pFileDir, pResolveOptions) {
    const lRetval = determineNpmDependencyTypes(localNpmHelpers.getPackageRoot(pModuleName), pPackageDeps);
    if (localNpmHelpers.dependencyIsDeprecated(pModuleName, pFileDir, pResolveOptions)) {
        lRetval.push("deprecated");
    }
    if (dependencyIsBundled(pModuleName, pPackageDeps)) {
        lRetval.push("npm-bundled");
    }
    return lRetval;
}
function isNodeModule(pDependency) {
    return !!pDependency.resolved && pDependency.resolved.includes("node_modules");
}
function determineModuleDependencyTypes(pDependency, pModuleName, pPackageDeps, pFileDir, pResolveOptions) {
    if (isNodeModule(pDependency)) {
        return determineNodeModuleDependencyTypes(pModuleName, pPackageDeps, pFileDir, pResolveOptions);
    }
    else {
        return ["localmodule"];
    }
}
function isModule(dependency, modules = ["node_modules"], baseDir = ".") {
    return modules.some(moduleName => {
        if (path.isAbsolute(moduleName) && dependency.resolved) {
            return path.resolve(baseDir, ...dependency.resolved).startsWith(moduleName);
        }
        return !!dependency.resolved && dependency.resolved.includes(moduleName);
    });
}
function isLocal(pModuleName) {
    return pModuleName.startsWith(".");
}
function isAliased(pModuleName, pAliasObject) {
    return Object.keys(pAliasObject || {}).some(pAliasLHS => pModuleName.startsWith(pAliasLHS));
}
function isLikelyTSAliased(moduleName, resolved, tsConfig) {
    return tsConfig && !isLocal(moduleName) && resolved && !resolved.includes("node_modules");
}
function isAliassy(moduleName, dependency, resolveOption) {
    return isAliased(moduleName, resolveOption.alias) || isLikelyTSAliased(moduleName, dependency.resolved, resolveOption.tsConfig);
}
exports.default = (pDependency, pModuleName, pPackageDeps, pFileDir, pResolveOptions, pBaseDir) => {
    let lRetval = ["undetermined"];
    pResolveOptions = pResolveOptions || {};
    if (pDependency.couldNotResolve) {
        lRetval = ["unknown"];
    }
    else if (resolve_1.isCore(pModuleName)) {
        lRetval = ["core"];
    }
    else if (isLocal(pModuleName)) {
        lRetval = ["local"];
    }
    else if (isModule(pDependency, pResolveOptions.modules, pBaseDir)) {
        lRetval = determineModuleDependencyTypes(pDependency, pModuleName, pPackageDeps, pFileDir, pResolveOptions);
    }
    else if (isAliassy(pModuleName, pDependency, pResolveOptions)) {
        lRetval = ["aliased"];
    }
    return lRetval;
};
