"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const lodash_1 = require("lodash");
const path = require("path");
const enhancedResolve_1 = require("./enhancedResolve");
const isLocal = (pModule) => pModule.startsWith(".");
const isScoped = (pModule) => pModule.startsWith("@");
function getPackageRoot(pModule) {
    if (!Boolean(pModule) || isLocal(pModule)) {
        return pModule;
    }
    const lPathElements = pModule.split("/");
    if (isScoped(pModule)) {
        if (lPathElements.length === 1) {
            return pModule;
        }
        return `${lPathElements[0]}/${lPathElements[1]}`;
    }
    return lPathElements[0];
}
exports.getPackageRoot = getPackageRoot;
function bareGetPackageJson(pModule, pBaseDir, pResolveOptions) {
    let lRetval = null;
    try {
        const lPackageJsonFilename = enhancedResolve_1.resolve(path.join(getPackageRoot(pModule), "package.json"), pBaseDir ? pBaseDir : ".", pResolveOptions);
        lRetval = JSON.parse(fs.readFileSync(lPackageJsonFilename, "utf8"));
    }
    catch (e) {
    }
    return lRetval;
}
const getPackageJson = lodash_1.memoize(bareGetPackageJson, (pModule, pBaseDir) => `${pBaseDir}|${pModule}`);
exports.getPackageJson = getPackageJson;
function dependencyIsDeprecated(pModule, pBaseDir, pResolveOptions) {
    let lRetval = false;
    const lPackageJson = getPackageJson(pModule, pBaseDir, pResolveOptions);
    if (Boolean(lPackageJson)) {
        lRetval = lPackageJson.hasOwnProperty("deprecated") && lPackageJson.deprecated;
    }
    return lRetval;
}
exports.dependencyIsDeprecated = dependencyIsDeprecated;
function getLicense(pModule, pBaseDir, pResolveOptions) {
    let lRetval = "";
    const lPackageJson = getPackageJson(pModule, pBaseDir, pResolveOptions);
    if (Boolean(lPackageJson) && lPackageJson.hasOwnProperty("license") && typeof lPackageJson.license === "string") {
        lRetval = lPackageJson.license;
    }
    return lRetval;
}
exports.getLicense = getLicense;
